-- -----------------------------------------------------
-- Database QuizFaber x PostgreSQL , db version : 2
-- -----------------------------------------------------
-- Project : QuizFaber
-- Licence : MIT
-- Author  : Luca Galli
-- Email   : info@quizfaber.com
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Table QuizState
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizState (
  Code INTEGER NOT NULL,
  Description VARCHAR(30) NULL,
  PRIMARY KEY (Code));

INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 0,'Draft') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=0);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 1,'Completed') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=1);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 2,'Candidate') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=2);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 10,'Published') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=10);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 98,'Archived') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=98);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 99,'Deleted') AS tmp  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=99);

-- -----------------------------------------------------
-- Table Quizzes
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS Quizzes (
  QZ_ID SERIAL,
  quizname VARCHAR(255) NOT NULL,
  Title VARCHAR(1024) NOT NULL,
  IntroText TEXT NULL,
  EpiText TEXT NULL,
  NumQuestions INTEGER NOT NULL,
  Author VARCHAR(255) NULL,
  Argument VARCHAR(255) NULL,
  Season VARCHAR(255) NULL,
  State INT NOT NULL DEFAULT 0,
  Link VARCHAR(255) NULL,
  Properties VARCHAR(65535) NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  PRIMARY KEY (QZ_ID),
  CONSTRAINT FK_QuizState
    FOREIGN KEY (State)
    REFERENCES QuizState (Code));

-- -----------------------------------------------------
-- Table PersonRole
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS PersonRole (
  ID INTEGER NOT NULL,
  RoleName VARCHAR(80) NOT NULL,
  RoleDescription VARCHAR(500) NULL,
  IsEnabled BOOLEAN NOT NULL,
  PRIMARY KEY (ID));

-- -----------------------------------------------------
-- Table Person
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS Person (
  ID SERIAL,
  PersonName VARCHAR(255) NOT NULL,
  UserIdentity VARCHAR(255) NOT NULL,
  UserPassword VARCHAR(255) NOT NULL,
  UserRole INTEGER NOT NULL,
  Info TEXT NULL,
  IsEnabled BOOLEAN NOT NULL,
  Source VARCHAR(1) NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  DateLastAccess TIMESTAMP NULL,
  PRIMARY KEY (ID),
  UNIQUE KEY indexUserIdentity (UserIdentity),
  CONSTRAINT UserRole_FK
    FOREIGN KEY(UserRole) 
	REFERENCES PersonRole(ID));


-- -----------------------------------------------------
-- Table Categories
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS Categories (
  QZ_ID INTEGER NOT NULL,
  CategoryName VARCHAR(80) NOT NULL,
  Description TEXT NULL,
  MaxQuestForQuiz INTEGER NULL,
  IsRandom BOOLEAN NULL,
  PRIMARY KEY (CategoryName, QZ_ID),
  CONSTRAINT FK_CategoryQuiz
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table Questions
-- -----------------------------------------------------


CREATE TABLE IF NOT EXISTS Questions (
  QZ_ID INTEGER NOT NULL,
  Num INTEGER NOT NULL,
  CategoryName VARCHAR(80) NOT NULL,
  TextQuestion TEXT NOT NULL,
  QuestionName VARCHAR(256) NULL,
  Type INTEGER NOT NULL,
  Weight FLOAT NOT NULL,
  Include BOOLEAN NOT NULL,
  TextFormat INTEGER NOT NULL,
  SortAnswers BOOLEAN NOT NULL,
  TextAbove TEXT NULL,
  TextBelow TEXT NULL,
  Remark TEXT NULL,
  PutSeparator BOOLEAN NULL,
  Timeout INTEGER NULL,
  Html TEXT NULL,
  PRIMARY KEY (QZ_ID, Num),
  CONSTRAINT Quest_FK
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT Category_FK
    FOREIGN KEY (CategoryName , QZ_ID)
    REFERENCES Categories (CategoryName , QZ_ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);

CREATE INDEX IF NOT EXISTS Category_FK_idx ON Questions (CategoryName, QZ_ID);


-- -----------------------------------------------------
-- Table ChoiceAnswers
-- -----------------------------------------------------


CREATE TABLE IF NOT EXISTS ChoiceAnswers (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  TypeOfAns INTEGER NOT NULL,
  SingleAns BOOLEAN NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT ChoiceAns_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table ChoiceAnswerItems
-- -----------------------------------------------------


CREATE TABLE IF NOT EXISTS ChoiceAnswerItems (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  NumAns INTEGER NOT NULL,
  TextFormat INTEGER NOT NULL,
  Ans TEXT NOT NULL,
  RightAns BOOLEAN NOT NULL,
  Hint TEXT NULL,
  Rem TEXT NULL,
  PointsIn INTEGER NULL,
  PointsOut INTEGER NULL,
  Html TEXT NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT ChoiceItems_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES ChoiceAnswers (QZ_ID , NumQst)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table BooleanAnswerItems
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS BooleanAnswerItems (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  NumAns INTEGER NOT NULL,
  Sentence TEXT NOT NULL,
  TrueAns BOOLEAN NOT NULL,
  TextFormat INTEGER NOT NULL,
  Html TEXT NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT BoolAns_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table FillGapAnswers
-- -----------------------------------------------------


CREATE TABLE IF NOT EXISTS FillGapAnswers (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  Phrase TEXT NOT NULL,
  Type INTEGER NOT NULL,
  TextFormat INTEGER NOT NULL,
  Html TEXT NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT FillGap_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table MatchingAnswers
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS MatchingAnswers (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  Type INTEGER NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT Match_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table PairNames
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS PairNames (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  NumAns INTEGER NOT NULL,
  LeftItem TEXT NOT NULL,
  RightItem TEXT NOT NULL,
  TextFormat INTEGER NOT NULL,
  Html TEXT NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT PairNames_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES MatchingAnswers (QZ_ID , NumQst)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table CustomQuestionTypes
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS CustomQuestionTypes (
  QZ_ID INTEGER NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  Description TEXT NULL,
  Version INTEGER NULL,
  Subversion INTEGER NULL,
  IconData TEXT NULL,
  IncludeConfirmButton BOOLEAN NULL,
  ValuateRightWrong BOOLEAN NULL,
  MaxScore INTEGER NULL,
  MinScore INTEGER NULL,
  JscriptCode TEXT NULL,
  HtmlCode TEXT NULL,
  PRIMARY KEY (QZ_ID, CustomName),
  CONSTRAINT FK_CustomQuestionType_Quiz
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table CustomParameters
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS CustomParameters (
  QZ_ID INTEGER NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  ParamName VARCHAR(255) NOT NULL,
  IsEncrypt BOOLEAN NOT NULL,
  Type INTEGER NOT NULL,
  DefValue VARCHAR(255) NULL,
  PRIMARY KEY (QZ_ID, CustomName, ParamName),
  CONSTRAINT FK_Params
    FOREIGN KEY (QZ_ID , CustomName)
    REFERENCES CustomQuestionTypes (QZ_ID , CustomName)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table CustomHeadTags
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS CustomHeadTags (
  QZ_ID INTEGER NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  FileName VARCHAR(255) NOT NULL,
  Content TEXT NOT NULL,
  ContentType INTEGER NOT NULL,
  PRIMARY KEY (FileName, CustomName, QZ_ID),
  CONSTRAINT FK_CustomHeadTags
    FOREIGN KEY (QZ_ID , CustomName)
    REFERENCES CustomQuestionTypes (QZ_ID , CustomName)
    ON DELETE CASCADE
    ON UPDATE CASCADE);

CREATE INDEX IF NOT EXISTS FK_CustomHeadTags_idx ON CustomHeadTags (QZ_ID, CustomName);

-- -----------------------------------------------------
-- Table CustomAnswers
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS CustomAnswers (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  ParamName VARCHAR(255) NOT NULL,
  ParamValue VARCHAR(255) NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst, CustomName, ParamName),
  CONSTRAINT FK_CustomAns_To_Question
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT FK_CustomAns_To_CustomParams
    FOREIGN KEY (QZ_ID , CustomName , ParamName)
    REFERENCES CustomParameters (QZ_ID , CustomName , ParamName)
    ON DELETE CASCADE
    ON UPDATE CASCADE);


CREATE INDEX IF NOT EXISTS FK_CustomAns_To_CustomParams_idx ON CustomAnswers (QZ_ID, CustomName, ParamName);


-- -----------------------------------------------------
-- Table Slides
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS Slides (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  SlideName VARCHAR(80) NOT NULL,
  Content TEXT NOT NULL,
  TextFormat INTEGER NOT NULL,
  BeforeQst BOOLEAN NOT NULL,
  IsExternal BOOLEAN NOT NULL,
  HtmlFileName VARCHAR(255) NULL,
  SpeechFileName VARCHAR(255) NULL,
  Timeout INTEGER NULL,
  PRIMARY KEY (QZ_ID, NumQst, SlideName),
  CONSTRAINT FK_Slide_To_Question
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    ON UPDATE CASCADE);



-- -----------------------------------------------------
-- Table SlideDocs
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS SlideDocs (
  QZ_ID INTEGER NOT NULL,
  NumQst INTEGER NOT NULL,
  SlideName VARCHAR(80) NOT NULL,
  DocFileName VARCHAR(255) NOT NULL,
  LinkName VARCHAR(500) NOT NULL,
  PRIMARY KEY (DocFileName, QZ_ID, NumQst, SlideName),
  CONSTRAINT FK_Doc_To_Slide
    FOREIGN KEY (QZ_ID , NumQst , SlideName)
    REFERENCES Slides (QZ_ID , NumQst , SlideName)
    ON DELETE CASCADE
    ON UPDATE CASCADE);

CREATE INDEX IF NOT EXISTS FK_Doc_To_Slide_idx ON SlideDocs (QZ_ID, NumQst, SlideName);


-- -----------------------------------------------------
-- Table Quiz_Result
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizResult (
  ID SERIAL,
  QuizName VARCHAR(255) NOT NULL,
  QuizTitle VARCHAR(1024) NOT NULL,
  QuizID INTEGER NULL,
  UserName VARCHAR(255) NOT NULL,
  UserLogin VARCHAR(255) NOT NULL,
  DateCompleted TIMESTAMP NOT NULL,
  DateReceived TIMESTAMP NOT NULL,
  QuestsNum INTEGER NOT NULL,
  HighestMark INTEGER NOT NULL,
  LowestMark INTEGER NOT NULL DEFAULT 0,
  RoundMark REAL NOT NULL DEFAULT 1,
  FinalMark REAL NOT NULL,
  ReviewMark REAL NULL,
  ReviewDate TIMESTAMP NULL,
  FinalPoints REAL NULL,
  ReviewPoints REAL NULL,
  TotalTime INTEGER NOT NULL,
  ElapsedTime INTEGER NOT NULL,
  RightQuestsNum INTEGER NOT NULL,
  WrongQuestsNum INTEGER NOT NULL,
  NotAnsweredQuestsNum INTEGER NULL,
  NotValuatedQuestsNum INTEGER NULL,
  NumOfRetake INTEGER NOT NULL,
  IsDuplicated BOOLEAN NULL,
  PRIMARY KEY (ID));


-- -----------------------------------------------------
-- Table Quiz_Result_Report
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizResultReport (
  ID INTEGER NOT NULL,
  Origin TEXT NULL,
  Report TEXT NULL,
  Charset VARCHAR(255) NULL,
  Language VARCHAR(10) NULL,
  PRIMARY KEY (ID),
  CONSTRAINT FK_Report_QuizResult
    FOREIGN KEY (ID)
    REFERENCES QuizResult (ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);


-- -----------------------------------------------------
-- Table QuizResultQuestion
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizResultQuestion (
  ID SERIAL,
  IdQuizResult INTEGER NOT NULL,
  QuestNum INTEGER NOT NULL,
  QuestType INTEGER NOT NULL,
  Weight REAL NOT NULL,
  ShortTextQuestion VARCHAR(1000) NULL,
  Valid INTEGER NOT NULL,
  Score INTEGER NOT NULL,
  MaxScore INTEGER NOT NULL,
  MinScore INTEGER NOT NULL, 
  Points REAL NULL,
  Feedback VARCHAR(65535) NULL,
  IsCancelled BOOLEAN NULL,
  PRIMARY KEY (ID),
  CONSTRAINT FK_ResultQuestion_To_Result
    FOREIGN KEY (IdQuizResult)
    REFERENCES QuizResult (ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);


-- -----------------------------------------------------
-- Table QuizResultAnswer
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizResultAnswer (
  IdResultQuestion INTEGER NOT NULL,
  AnswerNum INTEGER NOT NULL,
  Choice VARCHAR(500) NOT NULL,
  Valuation VARCHAR(500) NOT NULL,
  IsGuess BOOLEAN NOT NULL,
  Score INTEGER NULL,
  AdditionalText VARCHAR(65535) NULL,
  ShortTextAnswer VARCHAR(1000) NULL,
  ShortTextRemark VARCHAR(1000) NULL,
  PRIMARY KEY (IdResultQuestion,AnswerNum),
  CONSTRAINT FK_ResultAnswer_To_ResultQuestion
    FOREIGN KEY (IdResultQuestion)
    REFERENCES QuizResultQuestion (ID)
    ON DELETE CASCADE
    ON UPDATE CASCADE);


-- -----------------------------------------------------
-- Table `QuizSession`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS QuizSession (
  PersonID INTEGER NOT NULL,
  SessionID INTEGER NOT NULL SERIAL,
  DateCreated TIMESTAMP NOT NULL,
  IpLogin VARCHAR(46) NULL,
  SessionData TEXT NULL,
  DateLastUpdate TIMESTAMP NULL,
  QuizID INTEGER NULL,
  IpLogout VARCHAR(46) NULL,
  UserAgent VARCHAR(255) NULL,
  IsRecoverable BOOLEAN NULL,
  PRIMARY KEY (SessionID),
  CONSTRAINT FK_QuizSessionPerson
    FOREIGN KEY (PersonID)
    REFERENCES Person (ID),
  CONSTRAINT FK_QuizSessionQuizResult
    FOREIGN KEY (QuizID)
    REFERENCES QuizResult (ID));

-- -----------------------------------------------------
-- Table Settings
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS Settings (
  DbVersion INTEGER NOT NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  ServerName VARCHAR(50) NULL,
  ServerDescription VARCHAR(50) NULL,
  ServerVersion VARCHAR(50) NULL,
  Environment VARCHAR(50) NULL,
  PortalBaseUrl VARCHAR(255) NULL,
  PhpMyAdminUrl VARCHAR(255) NULL,
  OwnerUrl VARCHAR(255) NULL,
  FooterText VARCHAR(255) NULL);

-- -----------------------------------------------------
-- Table WebLog
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS WebLog (
  ID INTEGER NOT NULL SERIAL,
  UserIdentity VARCHAR(255) NULL,
  SeverityLevel INTEGER NULL,
  DateCreated TIMESTAMP NULL,
  Message TEXT NULL,
  PRIMARY KEY (ID));

-- -----------------------------------------------------
-- Table WebSeverityLevel
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS WebSeverityLevel (
  Level INTEGER NOT NULL,
  Description VARCHAR(20) NULL);

DELETE FROM WebSeverityLevel;
INSERT INTO WebSeverityLevel (Level,Description) VALUES (0,'Debug');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (1,'Info');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (2,'Warning');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (3,'Error');

INSERT INTO Settings (DbVersion,DateCreated,DateModified,ServerName,ServerDescription,ServerVersion,Environment,PortalBaseUrl,PhpMyAdminUrl,OwnerUrl,FooterText) SELECT * FROM (SELECT 2,NOW() as date1,NOW() as date2,'Node.js server','Application for manage quizzes.','4.1.0','DEV','http://localhost:3313/',NULL,'https://www.quizfaber.com','QuizFaber' ) AS tmp WHERE NOT EXISTS (SELECT * FROM Settings);

INSERT INTO PersonRole (ID,RoleName,RoleDescription,IsEnabled) SELECT * FROM ( SELECT 1 AS ID,'admin' AS RoleName,'administrator role' AS RoleDescription,TRUE AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT RoleName FROM PersonRole WHERE RoleName = 'admin');
INSERT INTO PersonRole (ID,RoleName,RoleDescription,IsEnabled) SELECT * FROM ( SELECT 2 AS ID,'user' AS RoleName,'user role' AS RoleDescription,TRUE AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT RoleName FROM PersonRole WHERE RoleName = 'user');

-- password : admin
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Administrator' AS PersonName,'admin' AS UserIdentity,'21232f297a57a5a743894a0e4a801fc3' AS UserPassword,1 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'admin')
-- password : anonymous
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Anonymous' AS PersonName, 'anonymous@anonymous.nowhere' AS UserIdentity,'294de3557d9d00b3d2d8a1e6aab028cf' AS UserPassword,2 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'anonymous@anonymous.nowhere')
